////////////////////////////////////////////////////////////////////////////////
// Part of Internet EDiting component                                         //
// AUTHOR       : Sergey Kanenko                                              //
// WEBSITE      : http://iedcomp.nm.ru                                        //
////////////////////////////////////////////////////////////////////////////////
//   This code may be used and modified by anyone so long as  this header and //
// copyright  information remains intact.                                     //
//   The code is provided "as-is" and without warranty of any kind,           //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//   In no event shall the author be liable for any special, incidental,      //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
////////////////////////////////////////////////////////////////////////////////
unit UnMainDiagram;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls, ToolWin, ImgList,
  IdFTP, IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP,
  IEShapes, HREFParser, HREFLinker, UnParserShapes, Menus, ShellAPI,
  Printers, Clipbrd;
type
  TfrmMain = class(TForm)
    ControlBar1: TControlBar;
    ToolBar1: TToolBar;
    Diagram: TScrollBox;
    btnOpen: TToolButton;
    OpenDialog1: TOpenDialog;
    ToolButton1: TToolButton;
    DocImages: TImageList;
    LNK: THREFLinker;
    HTTPgate: TIdHTTP;
    FTPgate: TIdFTP;
    ToolButton5: TToolButton;
    StatusBar1: TStatusBar;
    edURL: TEdit;
    btnGO: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    btnStop: TToolButton;
    ToolButton8: TToolButton;
    LocalPopup: TPopupMenu;
    Showsourcecode1: TMenuItem;
    Shellexecute1: TMenuItem;
    ToolButton9: TToolButton;
    ToolButton10: TToolButton;
    PrintDialog1: TPrintDialog;
    btnCopy: TToolButton;
    ToolButton12: TToolButton;
    N1: TMenuItem;
    Delete1: TMenuItem;
    ToolButton11: TToolButton;
    SaveDialog1: TSaveDialog;
    TBImages: TImageList;
    procedure ToolButton1Click(Sender: TObject);
    procedure DiagramMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnOpenClick(Sender: TObject);
    procedure LNKDocumentParsed(Sender: TObject; Doc: THREFDocument;  LinksList: TStrings);
    procedure LNKNewDocument(Sender: TObject; Link: THREF);
    procedure btnGOClick(Sender: TObject);
    procedure ToolButton2Click(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure ToolButton8Click(Sender: TObject);
    procedure Showsourcecode1Click(Sender: TObject);
    procedure Shellexecute1Click(Sender: TObject);
    procedure ToolButton9Click(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure ToolButton11Click(Sender: TObject);
  private
    procedure CreateMetaDiagram(Diagram: TScrollBox; MF: TMetafile; Handle: HDC);
    procedure Shellexec(DocName: string);
  public
    procedure ShowSourceCodeFor(shp: TIEParserShape);
    procedure ShapeDblClick(Sender: TObject);
    procedure GO(URL: string);
    function PaintDiagramTo(Diagram: TScrollBox; ACanvas: TCanvas): Double;
    procedure PrintDiagram(Diagram: TScrollBox; Title: string; prn: TPrinter);
  end;

var
  frmMain: TfrmMain;
  CashList : TStringList;

implementation

uses UnData, UnConfig, Types, UnExport;

{$R *.DFM}

procedure TfrmMain.ToolButton1Click(Sender: TObject);
begin
 ClearParent(Diagram);
end;

procedure TfrmMain.DiagramMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 TIECustomShape.UnselectAllShapes(Diagram);
end;

procedure TfrmMain.btnOpenClick(Sender: TObject);
begin
if OpenDialog1.Execute then
 begin
  edURL.Text := OpenDialog1.FileName;
  GO(edURL.Text);
 end;
end;

procedure TfrmMain.ShowSourceCodeFor(shp: TIEParserShape);
begin
 if Assigned(shp) then
  With shp do
   begin
    if HREFdoc.HTML.Count=0 then
     begin
      ShowMessage('Source code hasn''t been loaded');
      frmData.Close;
      exit;
     end;
    frmData.MemoData.Lines.Text:=HREFdoc.HTML.Text;
    frmData.Caption := URL+' ['+HREFdoc.Title+']';
    frmData.Show;
   end;
end;

procedure TfrmMain.LNKDocumentParsed(Sender: TObject; Doc: THREFDocument;  LinksList: TStrings);
var i : integer;
    SrcShape : TIEParserShape;
begin
 SrcShape:=ShapeOf(Diagram,Doc.HREFself.UniqueDocURL);
 SrcShape.HREFdoc.Assign(Doc);  //Store document sources

 if frmConfig.rgDocumentCaption.ItemIndex=3 then
  begin
   SrcShape.ShapeCaption.Text := Doc.Title;
   SrcShape.AlignCaption;
  end;

 for i:=0 to LinksList.Count-1 do
   AddLink( Diagram,
            SrcShape,
            ShapeOf(Diagram, THREF(LinksList.Objects[i]).UniqueDocURL),
            TColor($008888DD), //clBroken,
            TColor($00DADADA), //clUnchecked,
            TColor($00DD3333), //clRemote,
            clGray             //clLocal
             );
end;

procedure TfrmMain.LNKNewDocument(Sender: TObject; Link: THREF);
var shp   : TIECustomShape;
    Title : string;
begin
  Case frmConfig.rgDocumentCaption.ItemIndex of
   0 : Title := Link.Document;
   1 : Title := Link.UniqueDocURL;
   2 : Title := Link.HREF;
   3 : Title := ' ';// Havn't title here
   4 : Title := '';
  end;
  shp := AddShape(Diagram,DocImages,Link, Title);
  if Assigned(shp) then
   With shp do
    begin
     OnDblClick:= ShapeDblClick;
     PopupMenu := LocalPopup;
    end;
end;

procedure TfrmMain.ShapeDblClick(Sender: TObject);
begin
 ShowSourceCodeFor(TIEParserShape(Sender));
end;

procedure TfrmMain.btnGOClick(Sender: TObject);
begin
 GO(edURL.Text);
end;

procedure TfrmMain.GO(URL:string);
begin
  ClearParent(Diagram);
  Diagram.Cursor := crHourGlass;
 try
  LNK.ParseFileByURL(URL);
 finally
  Diagram.Cursor := crDefault;
  Diagram.Invalidate;
 end;
end;

procedure TfrmMain.ToolButton2Click(Sender: TObject);
begin
 frmConfig.ShowModal;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
 LNK.DoTerminate := true;
 HTTPgate.Disconnect;
 FTPgate.Disconnect;
end;

procedure TfrmMain.Showsourcecode1Click(Sender: TObject);
var shp : TIECustomShape;
begin
 ShowSourceCodeFor(GetSelectedShape(Diagram));
end;

procedure TfrmMain.Shellexec(DocName : string);
begin
  ShellExecute(Application.Handle, 'open', PChar(DocName), nil, nil, SW_SHOWNORMAL);
end;

procedure TfrmMain.Shellexecute1Click(Sender: TObject);
var shp : TIEParserShape;
begin
 shp:= GetSelectedShape(Diagram);
 if Assigned(shp) then
  Shellexec(shp.HREFdoc.HREFSelf.HREF);
end;

procedure TfrmMain.Delete1Click(Sender: TObject);
var shp : TIEParserShape;
begin
 shp:= GetSelectedShape(Diagram);
 if Assigned(shp) then
  shp.Free;
end;

procedure TfrmMain.ToolButton8Click(Sender: TObject);
begin
ShowMessage('HREFparser '+IEDVersion+' demo '+#13#13+
            'Select any  remote  or  local document. Click go'+#13+
            'button and  You see document  link  structure on'+#13+
            'diagram below. Double click on selected document'+#13+
            'will show document source code.'+#13#13+
            'Main   site http://iedcomp.nm.ru'+#13+
            'Mirror site http://iedcomp.happytocode.com'+#13#13+
            'You comments and proposition mail to iedcomp@nm.ru');
end;

procedure TfrmMain.CreateMetaDiagram(Diagram: TScrollBox; MF:TMetafile; Handle: HDC);
Var MC : TMetafileCanvas;
    i  : integer;
begin
  MF.Width  := Diagram.HorzScrollBar.Range;
  MF.Height := Diagram.VertScrollBar.Range;
  MC := TMetafileCanvas.Create(MF, Handle);
  try
   for i:=0 to Diagram.ControlCount-1 do
    if Diagram.Controls[i] is TIECustomShape then
      TIECustomShape(Diagram.Controls[i]).PaintToCanvas(MC);
  finally
   MC.Free;
  end;
end;

function TfrmMain.PaintDiagramTo(Diagram: TScrollBox; ACanvas: TCanvas): Double;
Var VW, VH, WW,WH : integer;
    MF : TMetafile;
begin
  With Diagram  do  begin
   WW:= HorzScrollBar.Range;
   WH:= VertScrollBar.Range;
  end;
  With ACanvas.ClipRect  do  begin
   VW:= Right-Left;
   VH:= Bottom-Top;
  end;
 // Fit to page
 if (WW>WH) then
   Result:= VW/WW
 else
   Result:= VH/WH;
 VW:= Round(WW*Result);
 VH:= Round(WH*Result);
 try
  MF := TMetafile.Create;
  CreateMetaDiagram(Diagram, MF, ACanvas.Handle);
  ACanvas.StretchDraw(Rect(0,0,VW,VH),MF);
 finally
  MF.Free;
 end;
end;

procedure TfrmMain.PrintDiagram(Diagram: TScrollBox; Title: string; prn: TPrinter);
begin
  Prn.Title:=Title;
  Prn.BeginDoc;
  try
   PaintDiagramTo(Diagram,prn.Canvas);
  finally
   Prn.EndDoc;
  end;
end;

procedure TfrmMain.ToolButton9Click(Sender: TObject);
begin
if PrintDialog1.Execute then
  PrintDiagram(Diagram, 'HREFparser '+IEDVersion+' demo', Printer);
end;

procedure TfrmMain.btnCopyClick(Sender: TObject);
Var
    MF   : TMetafile;
    Fmt  : Word;
    Data : THandle;
    Pal  : HPALETTE;
begin
 try
  MF := TMetafile.Create;
  MF.Enhanced := false;
  CreateMetaDiagram(Diagram, MF, Self.Canvas.Handle);
  MF.SaveToClipboardFormat(Fmt,Data,Pal);
  ClipBoard.SetAsHandle(Fmt,Data);
 finally
  MF.Free;
 end;
end;

procedure TfrmMain.ToolButton11Click(Sender: TObject);
var Img   : TBitMap;
    Scale : double;
begin

  With Diagram  do  begin
   frmExport.udWidth.Position  := HorzScrollBar.Range;
   frmExport.udHeight.Position := VertScrollBar.Range;
  end;

 if frmExport.ShowModal = mrOK then
  if SaveDialog1.Execute then
   begin
    Img:=frmExport.CreateNewMap;
    Scale:=PaintDiagramTo(Diagram,Img.Canvas);
    frmExport.ExportToHTML(Diagram,SaveDialog1.FileName,Scale);
    Shellexec(SaveDialog1.FileName);
   end;
end;

end.

